//Main Sketch - Version 2.1 - www.cheap-thermocam.tk

#include <Servo.h>
#include <i2cmaster.h>
#include <EEPROM.h>


//Don't change those values
char st1[30];
long int count = 0;
int mlr;
int mud;
int blr;
int bud;
int lines = 48;
int rows = 64;
char c = 0;

Servo lr; //Servo for left-right movement
Servo ud; //Servo for up-down movement
//Adjust those values if neccesary
int lrpin = 9; //Left-Right Servo Pin
int udpin = 8; //Up-Down Servo Pin

void setup(){ 
 Serial.begin(115200); //BaudRate for Serial Communication
  mlr = EEPROM.read(0)*10; //Read existing calibration from the EEPROM
  mud = EEPROM.read(1)*10;
  if(mlr == 0 || mud == 0){
    mlr = 1275; //Middle point for LR Servo; only used if there is no existing calibration
    mud = 1750; //Middle point for UD Servo; only used if there is no existing calibration
  }
  blr = EEPROM.read(2)*10;
  bud = EEPROM.read(3)*10;
  if(blr == 0 || bud == 0){
    blr = 1600; //Left-Bottom point for LR Servo; only used if there is no existing calibration
    bud = 1550; //Left-Bottom point for UD Servo; only used if there is no existing calibration
  }

 PORTC = (1 << PORTC4) | (1 << PORTC5); //Enabling Pull-Ups for the MLX90614
 
  ud.attach(udpin); //Attach servos
  lr.attach(lrpin); 
  
  ud.writeMicroseconds(mud); //Move servos to middle position
  lr.writeMicroseconds(mlr);  
}

void loop(){
if( Serial.available() > 0 )
    {
        c = Serial.read();
        switch( c )
        {
            case 'a':
                measure();
            break;
            case 'b':
                middleleft();
            break;
            case 'c':
                middleright();
            break;
            case 'd':
                middleup();
            break;
            case 'e':
                middledown();
            break;
            case 'f':
                middlesave();
            break;
            case 'g':
                bottomleft();
            break;
            case 'h':
                bottomright();
            break;
            case 'i':
                bottomup();
            break;
            case 'j':
                bottomdown();
            break;
            case 'k':
                bottomsave();
            break;
            case 'l':
                gotomiddle();
            break;
            case 'm':
                gotobottom();
            break;
            case 'n':
                res1();
            break;
            case 'o':
                res2();
            break;
            case 'p':
                res3();
            break;
            case 'q':
                res4();
            case 'r':
                livetemp();
            break;
        }
    }
}

void middleleft(){ //move middle pos left
   mlr += 10;
   lr.writeMicroseconds(mlr);
   ud.writeMicroseconds(mud);
 }
void middleright(){ //move middle pos right
   mlr -= 10;
   lr.writeMicroseconds(mlr);
   ud.writeMicroseconds(mud);
 }
void middleup(){ // move middle pos up
   mud += 10;
   ud.writeMicroseconds(mud);
   lr.writeMicroseconds(mlr);
 }
 void middledown(){ //move middle pos down
   mud -= 10;
   ud.writeMicroseconds(mud);
   lr.writeMicroseconds(mlr);
 }
void middlesave(){ //save middle pos to EEPROM
   EEPROM.write(0,(mlr*0.1));
   EEPROM.write(1,(mud*0.1));
 }
void bottomleft(){ //move bottom-left pos left
   blr += 10;
   lr.writeMicroseconds(blr);
   ud.writeMicroseconds(bud);
 }
void bottomright(){ //move bottom-left pos right
   blr -= 10;
   lr.writeMicroseconds(blr);
   ud.writeMicroseconds(bud);
 } 
void bottomup(){ //move bottom-left pos up
   bud += 10;
   ud.writeMicroseconds(bud);
   lr.writeMicroseconds(blr);
 }
void bottomdown(){ //move bottom-left pos down
   bud -= 10;
   ud.writeMicroseconds(bud);
   lr.writeMicroseconds(blr);
 }
void bottomsave(){ //save bottom-left pos to EEPROM
  EEPROM.write(2,(blr*0.1));
  EEPROM.write(3,(bud*0.1));
 }
void gotomiddle(){ //go to middle pos
   ud.writeMicroseconds(mud);
   lr.writeMicroseconds(mlr);
 }
void gotobottom(){ //go to bottom-left pos
   ud.writeMicroseconds(bud);
   lr.writeMicroseconds(blr);
 }
void res1(){ //set resolution to 64x48
   rows = 64;
   lines = 48;
 }
void res2(){ //set resolution to 48x36
   rows = 48;
   lines = 36;
 }
void res3(){ //set resolution to 32x24
   rows = 32;
   lines = 24;
 }
void res4(){ //set resolution to 16x12
   rows = 16;
   lines = 12;
 }
 
void livetemp(){
  while(true){
   long int tpl;
   char strcount[5];
   itoa (count, strcount, 10);
   tpl = readMLXtemperature(0); // read sensor object temperature
   tpl = tpl *10;
   tpl = tpl / 5;
   tpl=tpl-27315;
   char str1[5];
     strcpy (str1, "");
     char str2[10] = "%li.%li";
     strcat (str1, str2); 
     sprintf(st1,str1, tpl/100, abs(tpl %100) );
   Serial.print(st1);
   delay(200);
  }
}

void measure(){
  int ypos = bud;
  ud.writeMicroseconds(ypos);
  int xpos = blr;
  lr.writeMicroseconds(xpos);
  delay(50);
  readMLXtemperature(0);
  delay(50);
  for(int x=0;x<rows;x++){
        for(int y=0;y<lines;y++){
           temperatur(); //measure temperature at the specific point
           if(y != lines-1){
           ypos += (int) ((mud-bud)*2)/(lines-1);
           ud.writeMicroseconds(ypos);
           }
           delay(50);
        }
      for(int i=0;i<(lines+1);i++){
      ypos -= (int) ((mud-bud)*2)/(lines-1);
      ud.writeMicroseconds(ypos);
      delay(5);
      } 
      if(x != rows-1){  
        xpos -= (int) ((blr-mlr)*2)/(rows-1);
        lr.writeMicroseconds(xpos);
      } 
      readMLXtemperature(0);
      ypos += (int) ((mud-bud)*2)/(lines-1);
      delay(50);
      readMLXtemperature(0);
      ypos += (int) ((mud-bud)*2)/(lines-1);
      delay(50);
    }
    finish();
}

//do not change
long int readMLXtemperature(int TaTo) {
  long int lii;
  int dlsb,dmsb,pec;
  int dev = 0x5A<<1;
  i2c_init();
  i2c_start_wait(dev+I2C_WRITE);
  if (TaTo) i2c_write(0x06); else i2c_write(0x07);
  i2c_rep_start(dev+I2C_READ);
  dlsb = i2c_readAck();
  dmsb = i2c_readAck();
  pec = i2c_readNak();
  i2c_stop();
  lii=dmsb*0x100+dlsb;
  return(lii);
}

//do not change
int getAmbientTemp(){
  long int tpl;
  tpl=readMLXtemperature(1);
  tpl = tpl *10;
  tpl = tpl / 5;
  tpl=tpl-27315;
  return(tpl/100);
}


void temperatur(){
   count++;
   long int tpl;
   char strcount[5];
   itoa (count, strcount, 10);
   tpl = readMLXtemperature(0); // read sensor object temperature
   tpl = tpl *10;
   tpl = tpl / 5;
   tpl=tpl-27315;
   //Serial communication
   if(count < 10){
     char str1[5];
     strcpy (str1, "000");
     strcat (str1, strcount); 
     char str2[10] = "%03li.%li";
     strcat (str1, str2); 
     sprintf(st1,str1, tpl/100, abs(tpl %100) );
   }
   else if((count >= 10) && (count < 100)){
     char str1[5];
     strcpy (str1, "00");
     strcat (str1, strcount); 
     char str2[10] = "%03li.%li";
     strcat (str1, str2); 
     sprintf(st1,str1, tpl/100, abs(tpl %100) );
   }
   else if((count >= 100) && (count < 1000)){
           char str1[5];
     strcpy (str1, "0");
     strcat (str1, strcount); 
     char str2[10] = "%03li.%li";
     strcat (str1, str2); 
     sprintf(st1,str1, tpl/100, abs(tpl %100) );
   }
   else if(count >= 1000){
     char str1[5];
     strcpy (str1, "");
     strcat (str1, strcount); 
     char str2[10] = "%03li.%li";
     strcat (str1, str2); 
     sprintf(st1,str1, tpl/100, abs(tpl %100) );
   }   
   Serial.print(st1); //sent the temperature to the pc software    
}

void finish(){
  char roomtemp[3];
  itoa (getAmbientTemp(), roomtemp, 10);
  char roomoutput[4] = "r";
  strcat (roomoutput, roomtemp);
  Serial.print(roomoutput); //send the room temperature to the pc software
  delay(50);
  Serial.print("Stop");
  ud.writeMicroseconds(mud);
  lr.writeMicroseconds(mlr);
}

